// set the dimensions and margins of the graph
const bc_margin = {
    top: 50,
    right: 400,
    bottom: 45,
    left: 75,
  },
  bc_width = 945 - bc_margin.left - bc_margin.right,
  bc_height = 472 - bc_margin.top - bc_margin.bottom;

// append the svg object to the body of the page
const bc_svg = d3
  .select('#bubble_chart')
  .append('svg')
  .attr(
    'width',
    bc_width + bc_margin.left + bc_margin.right,
  )
  .attr(
    'height',
    bc_height + bc_margin.top + bc_margin.bottom,
  )
  .append('g')
  .attr(
    'transform',
    `translate(${bc_margin.left},${bc_margin.top})`,
  );

// read the data
d3.csv('bubblechart.csv').then((data) => {
  // add chart title
  bc_svg
    .append('text')
    .attr('x', bc_width / 2)
    .attr('y', -20)
    .attr('text-anchor', 'middle')
    .style('font-size', '22px')
    .text(
      'Correlation Between Population Area Ratio, GDP per Capita, and Life Expectancy',
    );

  // add x axis
  const x = d3
    .scaleLinear()
    .domain([0, d3.max(data, (d) => +d.gdp_per_capita)])
    .range([0, bc_width]);
  bc_svg
    .append('g')
    .attr('transform', `translate(0, ${bc_height})`)
    .call(d3.axisBottom(x));

  // add x axis label
  bc_svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', bc_width / 2)
    .attr('y', bc_height + 40)
    .text('GDP per Capita');

  // add y axis
  const y = d3
    .scaleLinear()
    .domain([0, d3.max(data, (d) => +d.life_expectancy)])
    .range([bc_height, 0]);
  bc_svg.append('g').call(d3.axisLeft(y));

  // add y axis label:
  bc_svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', -bc_height / 2)
    .attr('y', -50)
    .text('Life Expectancy')
    .attr('transform', 'rotate(-90)');

  // add a scale for bubble size
  const z = d3
    .scaleLinear()
    .domain([
      0,
      d3.max(data, (d) => +d.population_area_ratio),
    ])
    .range([1, 40]);

  // add a scale for bubble color
  const countries = Array.from(data, (d) => d.country);
  const myColor = d3
    .scaleOrdinal(d3.schemeCategory10)
    .domain(countries);

  // ---------------------------//
  //      tooltip               //
  // ---------------------------//

  // -1- create a tooltip div that is hidden by default:
  const tooltip = d3
    .select('#bubble_chart')
    .append('div')
    .style('opacity', 0)
    .attr('class', 'tooltip')
    .style('background-color', 'black')
    .style('border-radius', '5px')
    .style('padding', '10px')
    .style('color', 'white')
    .style('font-family', 'sans-serif');

  // -2- create 3 functions to show / update (when mouse moves but stay on same circle) / hide the tooltip
  const showTooltip = (event, d) => {
    tooltip.transition().duration(200);
    tooltip
      .style('opacity', 1)
      .html(
        `<strong>Country:</strong> ${d.country}<br><strong>GDP per Capita:</strong> ${d.gdp_per_capita}<br><strong>Life Expectancy:</strong> ${d.life_expectancy}<br><strong>Population Area Ratio:</strong> ${d.population_area_ratio}`,
      )
      .style('left', event.x + 25 + 'px')
      .style('top', event.y + 25 + 'px');
  };
  const moveTooltip = (event, d) => {
    tooltip
      .style('left', event.x + 25 + 'px')
      .style('top', event.y + 25 + 'px');
  };
  const hideTooltip = (event, d) => {
    tooltip.transition().duration(200).style('opacity', 0);
  };

  // add dots
  bc_svg
    .append('g')
    .selectAll('dot')
    .data(data)
    .join('circle')
    .attr('class', 'bubbles')
    .attr('cx', (d) => x(d.gdp_per_capita))
    .attr('cy', (d) => y(d.life_expectancy))
    .attr('r', (d) => z(d.population_area_ratio))
    .style('fill', (d) => myColor(d.country))
    .attr('stroke', 'black')
    .on('mouseover', showTooltip)
    .on('mousemove', moveTooltip)
    .on('mouseleave', hideTooltip);

  // ---------------------------//
  //       legend              //
  // ---------------------------//

  // add one dot in the legend for each name
  const size = 5;
  const allgroups = countries;
  bc_svg
    .selectAll('myrect')
    .data(allgroups)
    .join('circle')
    .attr('cx', 550)
    .attr('cy', (d, i) => 6 + i * (size + 7))
    .attr('r', 3)
    .style('fill', (d) => myColor(d));

  // add labels beside legend dots
  bc_svg
    .selectAll('mylabels')
    .data(allgroups)
    .enter()
    .append('text')
    .attr('x', 550 + size * 2)
    .attr('y', (d, i) => 10 + i * (size + 7))
    .style('fill', (d) => myColor(d))
    .text((d) => d)
    .attr('text-anchor', 'left')
    .style('alignment-baseline', 'middle')
    .style('font-size', '13px');
});
